<?php

namespace App\Service;

use App\Dto\DtoCollection;
use App\Dto\DtoInterface;
use App\Dto\NewsCategoryDto;
use App\Dto\NewsFilterVariants;
use App\Dto\NewsListDto;
use App\Dto\NewsListingElementDto;
use App\Dto\PaginationDto;
use App\Entity\News;
use App\Entity\NewsCategories;
use App\Repository\NewsRepositoryInterface;
use App\Repository\NewsCategoriesRepositoryInterface;
use App\Request\AbstractRequest;

class NewsListingService implements ServiceInterface
{

    public function __construct(
        private readonly NewsRepositoryInterface $news,
        private readonly NewsCategoriesRepositoryInterface $newsCategories,
    ) {}

    public function serve(AbstractRequest $request): DtoInterface
    {
        $countOfRestaurants = $this
            ->news->
            getCountWithFilters($request->news_category);

        $pagination = new PaginationDto(
            $request->page,
            $request->limit,
            $countOfRestaurants
        );

        $offset = min($pagination->page_size, $countOfRestaurants) * ($pagination->current_page - 1);

        $list = new DtoCollection(
            NewsListingElementDto::class,
            $this
                ->news
                ->findByFilters(
                    $request->news_category,
                    $request->limit, $offset
                )
                ->map(function(News $news) {
                    return $news->getDto();
                })
                ->toArray()
        );

        $filters = new NewsFilterVariants(
            new DtoCollection(
                NewsCategoryDto::class,
                $this->newsCategories->getAll()
                    ->map(function(NewsCategories $category) {
                        return $category->getDto();
                    })
                    ->toArray()
            ),
        );

        return new NewsListDto(
            $pagination,
            $list,
            $filters
        );
    }
}
